#include "partition.hpp"
#include <string>
#include <iostream>
#include <iomanip>

using namespace std;
using namespace partition;

// This sample program shows how to use a partition to track the states of a
// box of 8 crayons being shared by two children, Becky and Cate, to draw
// pictures.

// Define an enum Color that defines the 8 crayon colors.  I'll also define
// one special value NONE_AVAILABLE to represent situations where a desired
// crayon type is not available.
//
enum Color { NONE_AVAILABLE = -1, RED, ORANGE, YELLOW, GREEN, BLUE,
                                  VIOLET, BROWN, BLACK, NUM_CRAYONS };
// Map each color to a string name:
//
string crayonName[] = { "red", "orange", "yellow", "green", "blue",
                        "violet", "brown", "black" };

// Each crayon can be in one of three User states:
//    in the box,
//    in use by Cate, or
//    in use by Becky
//
enum User { BOX, CATE, BECKY, NUM_USER };
string userName[] = { "Box", "Cate", "Becky" };

// To make things a little more interesting, we'll give each crayon
// a quality state as well:
//
enum Quality { SHARP, DULL, NUM_QUALITY };
string qualityName[] = { "Sharp", "Dull" };

// Define a simple structure to model the compound state of each crayon.
//
class CrayonState {
    private:
        User user;
        Quality quality;
    public:
        CrayonState() {}
        CrayonState(User user, Quality quality)
            :   user(user), quality(quality) {
        }
        User getUser() const { return user; }
        Quality getQuality() const { return quality; }
        friend ostream& operator << (ostream& os, const CrayonState& cs);
};

inline ostream& operator << (ostream& os, const CrayonState& cs) {
    return os << "(" << setw(5) << userName[cs.user] << ", " <<
                 setw(5) << qualityName[cs.quality] << ")";
}

// Define a typedef for a CrayonGroup to shorten some declartion syntax.
//
typedef Group<string,CrayonState> CrayonGroup;

// stream out a CrayonGroup.
//
inline ostream& operator << (ostream& os, const CrayonGroup& g) {
    os << g.getValue() << ":";
    for(CrayonGroup::ConstIterator i = g.front(); !i.isAfterBack(); ++i)
        os << " " << (*i).member;
    return os;
}

// This CrayonManager class tracks the state of each crayon via partition.
//
class CrayonManager {
    private:
        Domain<string,CrayonState> crayons;
        CrayonGroup state[NUM_USER][NUM_QUALITY];

    public:
        CrayonManager() {

            // Initialize the domain by adding the 8 crayons to it.
            //
            for(int i = 0; i < NUM_CRAYONS; ++i)
                crayons.addEntry(i, crayonName[i]);

            // Initialize the groups by binding each group to the crayons
            // domain, and tagging each group with the CrayonState
            // it represents.
            //
            for(int u = 0; u < NUM_USER; ++u) {
                for(int q = 0; q < NUM_QUALITY; ++q) {
                    state[u][q].setDomain(crayons);
                    state[u][q].setValue(CrayonState(User(u),Quality(q)));
                }
            }

            // Initially all crayons are sharp and in the box
            //
            state[BOX][SHARP].addAll();
        }

        const CrayonState& getState(Color c) const {
            return crayons.getValue(c);
        }

        User getUser(Color c) const {
            return crayons.getValue(c).getUser();
        }

        Quality getQuality(Color c) const {
            return crayons.getValue(c).getQuality();
        }

        void setState(Color c, User u, Quality q) {
            if(c != NONE_AVAILABLE) {
                cout << CrayonState(u,q) << " <- " << getState(c) <<
                    ": " << crayonName[c] << endl;
                state[u][q].addBack(c);
            }
        }

        void setUser(Color c, User u) {
            if(c != NONE_AVAILABLE)
                setState(c, u, getQuality(c));
        }

        void setQuality(Color c, Quality q) {
            if(c != NONE_AVAILABLE)
                setState(c, getUser(c), q);
        }

        Color find(Quality q, User u = BOX) const {
            if(state[u][q].size() > 0)
                return (Color) state[u][q].peekFront().id;
            else
                return NONE_AVAILABLE;
        }

        Color findPreferred(Quality q = SHARP, User u = BOX) const {
            if(state[u][q].size() > 0)
                return (Color) state[u][q].peekFront().id;
            else
                return find(q == SHARP ? DULL : SHARP, u);
        }

        Color findPreferred(Color c, Quality q=SHARP, User u=BOX) const {
            if(getUser(c) == u)
                return c;
            else
                return findPreferred(q, u);
        }

        void moveAll(User from, User to, Quality q) {
            cout << CrayonState(to, q) << " <- " << state[from][q] << endl;
            state[to][q].addBack(state[from][q]);
        }

        void moveAll(User from, User to) {
            for(int q = 0; q < NUM_QUALITY; ++q)
                moveAll(from, to, Quality(q));
        }

        friend ostream& operator << (ostream& os, const CrayonManager& cm);
};

inline ostream& operator << (ostream& os, const CrayonManager& cm) {
    for(int u = 0; u < NUM_USER; ++u)
        for(int q = 0; q < NUM_QUALITY; ++q)
            os << cm.state[u][q] << endl;
    return os;
}

int main(int argc, char** argv) {
    CrayonManager manager;

    Color c;

    // Becky grabs three crayons, preferrably orange, blue, and a sharp.
    // She dulls the first two.
    //
    c = manager.findPreferred(ORANGE);
    manager.setState(c, BECKY, DULL);

    c = manager.findPreferred(BLUE);
    manager.setState(c, BECKY, DULL);

    c = manager.findPreferred();
    manager.setUser(c, BECKY);

    // Cate grabs two crayons, preferrably blue and green.
    // She dulls the first one.
    //
    c = manager.findPreferred(BLUE);
    manager.setState(c, CATE, DULL);

    c = manager.findPreferred(GREEN);
    manager.setUser(c, CATE);

    // Becky returns all her crayons to the box...
    //
    manager.moveAll(BECKY, BOX);

    // ...and then grabs a sharp one...
    //
    c = manager.find(SHARP);
    manager.setUser(c, BECKY);

    // ...and makes it dull...
    //
    manager.setQuality(c, DULL);

    // Cate returns her dullest crayon to the box.
    //
    c = manager.findPreferred(DULL, CATE);
    manager.setUser(c, BOX);

    // Cate notices the sharp ones are disappearing fast,
    // so she grabs all the remaining sharp ones...
    //
    manager.moveAll(BOX, CATE, SHARP);

    // Becky gets mad and shows dad.
    //
    cout << "\nFinal States:\n" << manager << endl;

    return 0;
}
