// partition:  a container for organizing a set of sequential integers
//             into non-overlapping groups.
// Version:    0.1.1
// Copyright 2013 Matthew T. Busche.
//
// This program is free software:  you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.

/** \mainpage Partition
 **
 ** A partition is a high performance container that organizes a sequential
 ** set of integer IDs into an arbitrary number of non-overlapping groups.
 ** Each integer can be associated with a user defined member of type M, and
 ** each group can be associated with a user defined value of type V.
 ** Together these associations enable mappings from objects of type M (with
 ** known IDs) to objects of type V, and conversely from objects of type V
 ** (with known groups) to a list of objects of type M.
 ** 
 ** A partition has not one, but two fundamental classes:  Domain<M,V> and
 ** Group<M,V>.  An instance of Domain maintains a resizable array of Entry
 ** objects, each Entry having a sequentially numbered ID paired with a
 ** user-provided member object of type M.  The Entry objects of a Domain may
 ** be partitioned among instances of the companion class Group<M,V> where
 ** each Group maintains Entry objects from a single Domain as a doubly linked
 ** list, and also has an association with a user-provided value object of
 ** type V.  Each Entry may exist in at most one Group and so a Domain along
 ** with its associated Groups form a partition (in the mathematical sense) of
 ** the Entry objects from a Domain.
 ** 
 ** The functionality of a partition includes these constant time operations
 ** which together distinguish it from other common containers:
 ** 
 **-   Given an ID, determine the Group to which the associated Entry belongs
 **    (if any).
 **-   Given a Group, find an Entry in that Group (if any).
 **-   Given an ID, move the Entry associated with that ID to a particular
 **    Group simultaneously removing it from the Group with which it was
 **    previously a member (if any).
 **
 ** Important Notes:
 ** ----------------
 **
 **- Normally, the range of IDs defined for a Domain is known to the
 **  application.  Accordingly, passing an out-of-range ID to most methods is
 **  treated as an error.  For example, the following will (perhaps
 **  surprisingly) produce an exception:
 **
 **      Domain<int, std::string> d;
 **      d.addEntry(1, "one");
 **      d.addEntry(2, "two");
 **      Group<int, std::string> g(d);
 **      g.addBack(2);
 **      Iterator i = g.find(3); // Raises exception because 3 is not in Domain
 **
 **  The only methods that don't follow this rule are those
 **  that dynamically resize the Domain to accomodate the given id; for
 **  example:  `Domain::addEntry(int id, const M& member);`
 **
 **- Methods defined in this package validate caller inputs and internal state
 **  and throw a PartitionException when errors are detected.  These guards do
 **  not, however, protect against the use of invalidated iterators, nor do
 **  they protect against null references passed as input.  (The former was
 **  deemed too costly and the latter rarely difficult to debug.)  Because a
 **  primary benefit of the partition is good performance, some users may find
 **  these protections undesirable.  To this end, it is possible to reduce or
 **  eliminate this processing by lowering the partition safety level
 **  preprocessor setting (PARTITION_SL).  The default setting is 3 which
 **  causes all validations to be included in the build.  See the comments in
 **  [partitionSafety.hpp](partitionSafety_8hpp_source.html) for details.
 **
 **- Because these validations can be compiled out of the software, it is
 **  important to treat a thrown PartitionException as a software bug and not
 **  as a lesser condition that can be programatically handled.
 **/

#ifndef PARTITION_HPP
#define PARTITION_HPP

#include <stdexcept>
#include <sstream>
#include "partitionSafety.hpp"

#ifndef __GNUG__
#define __PRETTY_FUNCTION__ ""
#endif

namespace partition
{
    class PartitionException : public std::runtime_error
    {
        public:
            explicit PartitionException (const std::string& what_arg);
    };

    inline void partitionThrow(const char* func, const char* file, int line, const char* msg) {
        std::ostringstream oss;
        oss <<
                "PartitionException:        " << msg  << "\n" <<
#ifdef __GNUG__
                "    thrown from function:  " << func << "\n" <<
#endif
                "    in file:               " << file << "\n" <<
                "    at line:               " << line << "\n";
        throw PartitionException(oss.str());
    }

    template<typename M, typename V>
    class Group;

    template<typename M, typename V>
    class Domain
    {
            // Forward declartions
            //
        private: class Node;
        public:  class Entry;

        private:

            int   _high;
            int   _low;
            int   _size;
            int   _min;

            // Unlike most array based containers, Domain does not require
            // that the first element be indexed by 0.  To avoid having to
            // subtract off the id value of the first array element when
            // indexing into the Node array, the first entry of the
            // dynamically allocated array of Nodes is pointed to by (_node +
            // _low).  In this way, _node[x] always gives the Node with id
            // x and the important method _getNode(int id) can be implemented
            // efficiently as:
            //
            //             return _node + id;
            // 
            // The _node offset is created at the time of array allocation:
            //
            //     _node = static_cast<Node*>(::operator new (sizeof(Node) * _size)) - _low;
            //
            // You'll also note this offset being accounted for when the
            // array is deleted:
            //
            //     ::operator delete (_node + _low);
            //
            Node* _node;

        public:

            class Entry
            {
                public:
                    const int id; ///< The integer id field.
                    M member;     ///< The user-provided member of type M.

                    Entry(int id);
                    Entry(int id, const M& member);
            };

            Domain(int size = 0, int min = 0);
            Domain(const Domain<M,V>& d);
            template<typename InputIterator> Domain(
                    InputIterator first, InputIterator last, int min = 0);

            virtual ~Domain();

            int size() const;
            int min() const;

            Group<M,V>* getGroup(int id);
            const Group<M,V>* getGroup(int id) const;

            V& getValue(int id);
            const V& getValue(int id) const;

            M& getMember(int id);
            const M& getMember(int id) const;
            void setMember(int id, const M& m);

            Entry& getEntry(int id);
            const Entry& getEntry(int id) const;

            void addEntry(int id, const M& member = M());

            void resize(int newSize);
            void resize(int newSize, int newMin);

            int high() const;
            int low() const;
            int capacity() const;
            void reserve(int high);
            void reserve(int high, int low);
            void compact();

        private:

            class Node
            {
                public:
                    Group<M,V>* _group;
                    Node* _prev;
                    Node* _next;
                    Entry _entry;

                    Node(int id);
                    Node(int id, const M& member);
                    Node(Group<M,V>* group, Node* prev, Node* next, int id);
                    Node(Group<M,V>* group, Node* prev, Node* next, int id,
                            const M& member);
            };

            void _transform(int newHigh, int newLow, int newSize, int newMin);
            void _initNodeArray(Node* newNode, int newLow, int newSize, int newMin);
            const Node* _getNode(int id) const;
            Node* _getNode(int id);
            void _verifySize   (const char* func, const char* file, int line, int size) const;
            void _verifyRange  (const char* func, const char* file, int line, int id) const;
            void _verifyHighLow(const char* func, const char* file, int line, int high, int low) const;
            void _verifyInGroup(const char* func, const char* file, int line, const Node* n) const;

            friend class Group<M,V>;
            friend class Group<M,V>::Iterator;
            friend class Group<M,V>::ConstIterator;
    };


    template<typename M, typename V>
    class Group
    {
            // Forward declartions and typedefs
            //
        public:  class Iterator;
        public:  class ConstIterator;
        public:  typedef typename Domain<M,V>::Entry Entry;
        private: typedef typename Domain<M,V>::Node Node;

        private:

            Domain<M,V>* _domain;
            Node _end;
            int _size;
            V _value;

        public:

            // Constructors
            //
            Group();
            Group(Domain<M,V>& d);
            Group(Domain<M,V>& d, const V& v);
            Group(Domain<M,V>& d, const Group& g);
            template<typename InputIterator> Group(
                Domain<M,V>& d,
                const InputIterator& first,
                const InputIterator& last);

            // Destructor
            //
            virtual ~Group();

            // Basic property access
            //
            Domain<M,V>* getDomain();
            const Domain<M,V>* getDomain() const;
            void setDomain(Domain<M,V>& d);

            V& getValue();
            const V& getValue() const;
            void setValue(const V& v);

            int size() const;
            bool contains(int id) const;

            // Direct Entry access
            //
            Entry& peekFront();
            const Entry& peekFront() const;
            Entry& peekBack();
            const Entry& peekBack() const;

            // Adders
            //
            void addFront(int id);
            void addBack(int id);
            void addFront(Group& g);
            void addBack(Group& g);
            void addAll();

            // Removers
            //
            Entry& removeFront();
            Entry& removeBack();
            Entry& remove(int id);
            void   removeAll();

            // Iterator generators
            //
            Iterator front();
            ConstIterator front() const;
            Iterator back();
            ConstIterator back() const;
            Iterator beforeFront();
            ConstIterator beforeFront() const;
            Iterator afterBack();
            ConstIterator afterBack() const;
            Iterator find(int id);
            ConstIterator find(int id) const;

            Iterator begin();
            ConstIterator begin() const;
            Iterator end();
            ConstIterator end() const;

        private:

            void _addAfter(Node* n1, Node* n2);
            Node* _remove(Node* node);
            void _verifyDomain       (const char* func, const char* file, int line) const;
            void _verifySameDomain   (const char* func, const char* file, int line, const Group<M,V>& g) const;
            void _verifyDiffDomain   (const char* func, const char* file, int line, const Group<M,V>& g) const;
            void _verifyNotEmpty     (const char* func, const char* file, int line) const;
            void _verifyInGroup      (const char* func, const char* file, int line, const Node* n) const;
            friend class Domain<M,V>;

            Group(const Group& g);                   ///< NOT IMPLEMENTED
            const Group& operator=(const Group& g);  ///< NOT IMPLEMENTED

        public:

            class Iterator
            {
                private:

                    Group* _group;
                    Node* _node;

                public:

                    Iterator();
                    Iterator(const Iterator& i);
                    Iterator& operator=(const Iterator& i);
                    Iterator& operator++();
                    Iterator& operator--();
                    Iterator operator++(int);
                    Iterator operator--(int);
                    bool isBeforeFront() const;
                    bool isAfterBack() const;
                    bool isAtFront() const;
                    bool isAtBack() const;
                    Entry& operator*() const;
                    Entry* operator->() const;
                    Group* getContainer() const;
                    bool operator==(const Iterator& i) const;
                    bool operator==(const ConstIterator& i) const;
                    bool operator!=(const Iterator& i) const;
                    bool operator!=(const ConstIterator& i) const;
                    void addBefore(int id) const;
                    void addAfter(int id) const;
                    Entry& removeAndInc();
                    Entry& removeAndDec();

                private:

                    Iterator(Group* group, Node* node);
                    void _verifyGroup(const char* func, const char* file, int line) const;
                    void _verifyNotAtEnd(const char* func, const char* file, int line) const;
                    void _verifyCompare(const char* func, const char* file, int line, const Iterator& i) const;
                    void _verifyCompare(const char* func, const char* file, int line, const ConstIterator& i) const;
                    friend class Group;
            };

            class ConstIterator
            {
                private:

                    const Group* _group;
                    const Node* _node;

                public:

                    ConstIterator();
                    ConstIterator(const Iterator& i);
                    ConstIterator(const ConstIterator& i);
                    ConstIterator& operator=(const Iterator& i);
                    ConstIterator& operator=(const ConstIterator& i);
                    ConstIterator& operator++();
                    ConstIterator& operator--();
                    ConstIterator operator++(int);
                    ConstIterator operator--(int);
                    bool isBeforeFront() const;
                    bool isAfterBack() const;
                    bool isAtFront() const;
                    bool isAtBack() const;
                    const Entry& operator*() const;
                    const Entry* operator->() const;
                    const Group* getContainer() const;
                    bool operator==(const Iterator& i) const;
                    bool operator==(const ConstIterator& i) const;
                    bool operator!=(const Iterator& i) const;
                    bool operator!=(const ConstIterator& i) const;

                private:

                    ConstIterator(const Group* group, const Node* node);
                    void _verifyGroup(const char* func, const char* file, int line) const;
                    void _verifyNotAtEnd(const char* func, const char* file, int line) const;
                    void _verifyCompare(const char* func, const char* file, int line, const Iterator& i) const;
                    void _verifyCompare(const char* func, const char* file, int line, const ConstIterator& i) const;
                    friend class Group;
            };

    };

#define PARTITION_VERIFY_SIZE(size)          _verifySize         (__PRETTY_FUNCTION__, __FILE__, __LINE__, size     )
#define PARTITION_VERIFY_RANGE(id)           _verifyRange        (__PRETTY_FUNCTION__, __FILE__, __LINE__, id       )
#define PARTITION_VERIFY_HIGH_LOW(high, low) _verifyHighLow      (__PRETTY_FUNCTION__, __FILE__, __LINE__, high, low)
#define PARTITION_VERIFY_DOMAIN()            _verifyDomain       (__PRETTY_FUNCTION__, __FILE__, __LINE__           )
#define PARTITION_VERIFY_SAME_DOMAIN(group)  _verifySameDomain   (__PRETTY_FUNCTION__, __FILE__, __LINE__, group    )
#define PARTITION_VERIFY_DIFF_DOMAIN(group)  _verifyDiffDomain   (__PRETTY_FUNCTION__, __FILE__, __LINE__, group    )
#define PARTITION_VERIFY_NOT_EMPTY()         _verifyNotEmpty     (__PRETTY_FUNCTION__, __FILE__, __LINE__           )
#define PARTITION_VERIFY_IN_GROUP(node)      _verifyInGroup      (__PRETTY_FUNCTION__, __FILE__, __LINE__, node     )
#define PARTITION_VERIFY_GROUP()             _verifyGroup        (__PRETTY_FUNCTION__, __FILE__, __LINE__           )
#define PARTITION_VERIFY_NOT_AT_END()        _verifyNotAtEnd     (__PRETTY_FUNCTION__, __FILE__, __LINE__           )
#define PARTITION_VERIFY_COMPARE(i)          _verifyCompare      (__PRETTY_FUNCTION__, __FILE__, __LINE__, i        )

// ================
//  IMPLEMENTATION
// ================

    /** \namespace partition
     **
     ** The namespace hosting all classes supporting partition functionality.
     **/

    /** \class PartitionException
     **
     ** Extension of std::runtime_error thrown for all errors detected by
     ** software entities defined in the partition namespace.
     **/

    /** Constructor taking a string containing text decribing
     ** the nature of the exceptional condition.
     **/
    inline PartitionException::PartitionException(const std::string& what_arg)
        : runtime_error(what_arg)
    {
    }


    /** \class Domain::Entry
     **
     ** A data pair consisting of an integer ID and a user-provided member of
     ** type M.
     **/

    /** Constructs an Entry with the given ID and a default member value.
     **/
    template<typename M, typename V>
    inline Domain<M,V>::Entry::Entry(int id)
        :   id(id)
    {
    }

    /** Constructs an Entry with the given ID and member.
     **/
    template<typename M, typename V>
    inline Domain<M,V>::Entry::Entry(int id, const M& member)
        :   id(id),
            member(member)
    {
    }

    /** \class Domain
     **
     ** A resizable array of Entry objects having sequentially numbered IDs
     ** which may be partitioned across an arbitrary number of associated
     ** Groups.
     **
     ** Read the [package summary](index.html) for introductory information
     ** about Domain and its related classes.
     **
     ** The ID of the first Entry object (being the Entry with the lowest
     ** numbered ID in the Domain's internal array) may have an arbitrary
     ** value and is tracked by `min()`.  The ID of the last Entry object
     ** (being the Entry with the highest numbered ID in the Domain's internal
     ** array) is given by `min() + size() - 1`.  This internal array may be
     ** dynamically resized in either direction (either by adding entries
     ** after the end of the array which increases the maximum ID or by adding
     ** entries before the beginning of the array which decreases the minimum
     ** ID).
     **
     ** The methods `addEntry()`, `compact()`, `reserve()`, and `resize()`
     ** (and their overloaded variants) are the operations that can result in
     ** memory reallocation.  Memory reallocation does not disturb Group
     ** membership, but does invalidate all Iterators on all Groups.
     **
     ** \param M The Entry member type.
     ** \param V The Group value type.
     **/

    /** Constructs a Domain with `size` Entry objects having IDs ranging from
     ** `min` to `min + size - 1`.  All Entry member fields are assigned
     ** default values.
     **
     ** \throw PartitionException if `size` is negative.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_DOMAIN_CONSTRUCTOR
     ** \endcode
     **
     ** If size is zero, the value of min does not affect subsequent behavior
     ** of addEntry:  min always matches the id of the lowest numbered field
     ** in the Domain, so a subsequent call to addEntry with an id of any
     ** value N will always reset the min to N.  The min value is set however,
     ** so a subsequent call to resize(int size) will function with the
     ** previously set min value.
     **/
    template<typename M, typename V>
    inline Domain<M,V>::Domain(int size, int min)
        :   _high(0),
            _low(0),
            _size(0),
            _min(0),
            _node(NULL)
    {
        PARTITION_SAFETY_DOMAIN_CONSTRUCTOR(PARTITION_VERIFY_SIZE(size));
        _transform(min + size, min, size, min);
    }

    /** Constructs a Domain that is a copy of `d`.  The capacity of the new
     ** Domain matches the size of the original (minimizing storage
     ** requirements for the copy).</p>
     **/
    template<typename M, typename V>
    inline Domain<M,V>::Domain(const Domain<M,V>& d)
        :   _high(d.min() + d.size()),
            _low(d.min()),
            _size(d.size()),
            _min(d.min())
    {
        _node = static_cast<Node*>(::operator new (sizeof(Node) * _size)) - _low;
        Node* n = _node + _min;
        const Node* s = d._getNode(_min);
        for(int i = _min; i < _min + _size; ++i)
            new (n++) Node(i, (s++)->_entry.member);
    }

    /** Iterates from first to last and stores a copy of each element in the
     ** sequence as a member value of an Entry in this Domain.  The first
     ** entry is given an ID of min and each subsequent entry is given a
     ** sequentially increasing integer `id`.
     **/
    template<typename M, typename V> template<typename InputIterator>
    inline Domain<M,V>::Domain(InputIterator first, InputIterator last, int min)
        :   _high(0),
            _low(0),
            _size(0),
            _min(min),
            _node(NULL)
    {
        int id = min;
        for(InputIterator i = first; i != last; ++i)
            addEntry(id++, *i);
    }

    /** Destructs the Domain, removing all Entry objects from all Groups,
     ** calling the destructor for each Entry member, and deallocating all
     ** memory currently allocated for internal storage.
     **/
    template<typename M, typename V>
    inline Domain<M,V>::~Domain()
    {
        resize(0);
        if(capacity())
            ::operator delete (_node + _low);
    }

    /** Returns the number of Entry objects in this Domain.
     **/
    template<typename M, typename V>
    inline int Domain<M,V>::size() const
    {
        return _size;
    }

    /** Returns the lowest numbered ID of this Domain.
     **/
    template<typename M, typename V>
    inline int Domain<M,V>::min() const
    {
        return _min;
    }

    /** Returns a pointer to the Group to which the Entry with the given `id`
     ** belongs or `NULL` if that member belongs to no Group.
     **
     ** \throw PartitionException if `id` is less than `min()` or greater than
     ** `min() + size() - 1`.\n\n
     ** This validation is only included if the following
     ** preprocessor directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_DOMAIN_ACCESSOR
     ** \endcode
     **/
    template<typename M, typename V>
    inline Group<M,V>* Domain<M,V>::getGroup(int id)
    {
        PARTITION_SAFETY_DOMAIN_ACCESSOR(PARTITION_VERIFY_RANGE(id));
        return _getNode(id)->_group;
    }

    /** Returns a pointer to the const Group to which the Entry with the given
     ** `id` belongs or `NULL` if that member belongs to no Group.
     **
     ** \throw PartitionException if `id` is less than `min()` or greater than
     ** `min() + size() - 1`.\n\n
     ** This validation is only included if the following
     ** preprocessor directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_DOMAIN_ACCESSOR
     ** \endcode
     **/
    template<typename M, typename V>
    inline const Group<M,V>* Domain<M,V>::getGroup(int id) const
    {
        PARTITION_SAFETY_DOMAIN_ACCESSOR(PARTITION_VERIFY_RANGE(id));
        return _getNode(id)->_group;
    }

    /** Returns a reference to the value of the Group to which the Entry with
     ** the given `id` belongs.
     **
     ** \throw PartitionException if `id` is less than `min()` or greater than
     ** `min() + size() - 1`; or\n
     ** if the Entry with the given id belongs to no Group.\n\n
     ** These validations are only included if the following
     ** preprocessor directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_DOMAIN_ACCESSOR
     ** \endcode
     **/
    template<typename M, typename V>
    inline V& Domain<M,V>::getValue(int id)
    {
        PARTITION_SAFETY_DOMAIN_ACCESSOR(PARTITION_VERIFY_RANGE(id));
        Node* n = _getNode(id);
        PARTITION_SAFETY_DOMAIN_ACCESSOR(PARTITION_VERIFY_IN_GROUP(n));
        return n->_group->_value;
    }

    /** Returns a const reference to the value of the Group to which the Entry
     ** with the given `id` belongs.
     **
     ** \throw PartitionException if `id` is less than `min()` or greater than
     ** `min() + size() - 1`; or\n
     ** if the Entry with the given id belongs to no Group.\n\n
     ** These validations are only included if the following
     ** preprocessor directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_DOMAIN_ACCESSOR
     ** \endcode
     **/
    template<typename M, typename V>
    inline const V& Domain<M,V>::getValue(int id) const
    {
        PARTITION_SAFETY_DOMAIN_ACCESSOR(PARTITION_VERIFY_RANGE(id));
        const Node* n = _getNode(id);
        PARTITION_SAFETY_DOMAIN_ACCESSOR(PARTITION_VERIFY_IN_GROUP(n));
        return n->_group->_value;
    }

    /** Returns a reference to the member object associated with the given `id`.
     **
     ** Computational Complexity:  Constant.
     **
     ** \throw PartitionException if `id` is less than `min()` or greater than
     ** `min() + size() - 1`.\n\n
     ** This validation is only included if the following
     ** preprocessor directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_DOMAIN_ACCESSOR
     ** \endcode
     **/
    template<typename M, typename V>
    inline M& Domain<M,V>::getMember(int id)
    {
        PARTITION_SAFETY_DOMAIN_ACCESSOR(PARTITION_VERIFY_RANGE(id));
        return _getNode(id)->_entry.member;
    }

    /** Returns a const reference to the member object associated with the given `id`.
     **
     ** Computational Complexity:  Constant.
     **
     ** \throw PartitionException if `id` is less than `min()` or greater than
     ** `min() + size() - 1`.\n\n
     ** This validation is only included if the following
     ** preprocessor directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_DOMAIN_ACCESSOR
     ** \endcode
     **/
    template<typename M, typename V>
    inline const M& Domain<M,V>::getMember(int id) const
    {
        PARTITION_SAFETY_DOMAIN_ACCESSOR(PARTITION_VERIFY_RANGE(id));
        return _getNode(id)->_entry.member;
    }

    /** Finds the Entry in this Domain with the given `id` and sets its
     ** member field to `m`.
     **
     ** Computational Complexity:  Constant.
     **
     ** \throw PartitionException if `id` is less than `min()` or greater than
     ** `min() + size() - 1`.\n\n
     ** This validation is only included if the following
     ** preprocessor directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_DOMAIN_ACCESSOR
     ** \endcode
     **/
    template<typename M, typename V>
    inline void Domain<M,V>::setMember(int id, const M& m)
    {
        PARTITION_SAFETY_DOMAIN_ACCESSOR(PARTITION_VERIFY_RANGE(id));
        _getNode(id)->_entry.member = m;
    }

    /** Returns a reference to the Entry with the given `id`.
     **
     ** Computational Complexity:  Constant.
     **
     ** \throw PartitionException if `id` is less than `min()` or greater than
     ** `min() + size() - 1`.\n\n
     ** This validation is only included if the following
     ** preprocessor directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_DOMAIN_ACCESSOR
     ** \endcode
     **/
    template<typename M, typename V>
    inline typename Domain<M,V>::Entry& Domain<M,V>::getEntry(int id)
    {
        PARTITION_SAFETY_DOMAIN_ACCESSOR(PARTITION_VERIFY_RANGE(id));
        return _getNode(id)->_entry;
    }

    /** Returns a const reference to the Entry with the given `id`.
     **
     ** Computational Complexity:  Constant.
     **
     ** \throw PartitionException if `id` is less than `min()` or greater than
     ** `min() + size() - 1`.\n\n
     ** This validation is only included if the following
     ** preprocessor directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_DOMAIN_ACCESSOR
     ** \endcode
     **/
    template<typename M, typename V>
    inline const typename Domain<M,V>::Entry& Domain<M,V>::getEntry(int id) const
    {
        PARTITION_SAFETY_DOMAIN_ACCESSOR(PARTITION_VERIFY_RANGE(id));
        return _getNode(id)->_entry;
    }

    /** Similar to setMember but will resize this Domain as neccessary to
     ** accomodate the given `id`.  If the Domain has size zero or if `id` is
     ** less than the current `min`, then `min` is set to `id`.  If the Domain
     ** has capacity zero, then both `min` and `low` are set to `id`.
     **/
    template<typename M, typename V>
    inline void Domain<M,V>::addEntry(int id, const M& member)
    {
        // There are a few cases here.
        //
        // First, if the id already exists in the Domain, no call to
        // _transform is made and the member value of the existing entry is
        // simply over written to point to the new member value.
        //
        // Second, if id is either immediately before or after the existing
        // Domain range and there is sufficient reserved memory to construct
        // the new Node without memory reallocation, then no call to
        // _transform is made, the new Node is constructed locally, and size
        // and min are adjusted as appropriate.
        //
        // Otherwise a call is made to _transform is made (either because a
        // a memory reallocation is required, or because the id to be added
        // will result in the size of the Domain growing by more than one.)
        //
        // If a call to _transform is made, it is always made so that the
        // requested Node itself is not constructed, leaving the Domain
        // with a size one smaller than it will be once the entry is added.
        // In this way, if a Node must be constructed to hold the given Entry,
        // it is always constructed locally by this method.  (This is done
        // both to take advantage of the specialized constructor that sets the
        // member and also to unify the code path with the more typical case
        // where the size of the Domain is growing by one.)
        //
        // The subcase where the capacity is zero is handled differently and
        // results in the low and min values being reset to match the supplied
        // id.
        //
        if(!_size)
            _transform(id+1, id, 0, id);
        else if(id < _min - 1 || id < _low)
            _transform(_high, id, (_min + _size) - (id + 1), id + 1);
        else if(id > _min + _size || id >= _high)
            _transform(id+1, _low, id - _min, _min);
        if(id >= _min && id < _min + _size)
            _node[id]._entry.member = member;
        else
        {
            new (_node + id) Node(id, member);
            ++_size;
            if(id < _min)
                _min = id;
        }
    }

    /** Equivalent to resize(newSize, min())
     **/
    template<typename M, typename V>
    inline void Domain<M,V>::resize(int newSize)
    {
        resize(newSize, _min);
    }

    /** Resizes this Domain to contain newSize Entry objects with IDs
     ** ranging from `newMin` to `newMin + newSize - 1`.
     **
     ** Previously existing Entry objects having IDs that fall in this newly
     ** defined range have both their member fields as well as their group
     ** memberships preserved.
     **
     ** Previously existing Entry objects having IDs that fall outside this
     ** newly defined range are removed from their Groups, and each such Entry
     ** member's destructor is invoked.
     **
     ** If `newMin &lt; low()` or `newMin + newSize &gt; high()` then resize
     ** also causes a memory reallocation to accomodate the requested Entry
     ** range.
     **
     ** \throw PartitionException if `newSize` is negative.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_DOMAIN_MEMORY
     ** \endcode
     **/
    template<typename M, typename V>
    inline void Domain<M,V>::resize(int newSize, int newMin)
    {
        PARTITION_SAFETY_DOMAIN_MEMORY(PARTITION_VERIFY_SIZE(newSize));
        _transform(newMin + newSize, newMin, newSize, newMin);
    }

    /** Returns one plus the highest numbered ID that can be added to the
     ** Domain without memory reallocation.
     **/
    template<typename M, typename V>
    inline int Domain<M,V>::high() const
    {
        return _high;
    }

    /** Returns the lowest numbered ID that can be added to the Domain without
     ** memory reallocation.
     **/
    template<typename M, typename V>
    inline int Domain<M,V>::low() const
    {
        return _low;
    }

    /** Returns `high() - low()`.
     **/
    template<typename M, typename V>
    inline int Domain<M,V>::capacity() const
    {
        return _high - _low;
    }

    /** Equivalent to reserve(newHigh, low()).
     **/
    template<typename M, typename V>
    inline void Domain<M,V>::reserve(int newHigh)
    {
        // Just ignore calls where new high is less than current low.
        //
        if(_low > newHigh)
            return;
        _transform(newHigh, _low, _size, _min);
    }


    /** If either `high() < newHigh` or `low() > newLow`, then the internal
     ** array is reallocated so that `high()` is increased to at least
     ** `newHigh` and `low()` is reduced to at most `newLow`.  After the call,
     ** IDs from `newLow` to `newHigh - 1` may subsequently added to the
     ** Domain without additional reallocations.</p>
     **
     ** \throw PartitionException if `newHigh &lt; newLow`.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_DOMAIN_MEMORY
     ** \endcode
     **/
    template<typename M, typename V>
    inline void Domain<M,V>::reserve(int newHigh, int newLow)
    {
        PARTITION_SAFETY_DOMAIN_MEMORY(PARTITION_VERIFY_HIGH_LOW(newHigh, newLow));
        _transform(newHigh, newLow, _size, _min);
    }

    /** If the capacity of the Domain exceeds its size, then compact
     ** reallocates internal storage reducing capacity to match the
     ** current size of the Domain.
     **/
    template<typename M, typename V>
    inline void Domain<M,V>::compact()
    {
        if(capacity() == size())
            return;

        int newHigh = _min + _size;
        int newLow = _min;
        int newCapacity = newHigh - newLow;
        Node* newNode = static_cast<Node*> (::operator new (sizeof(Node) * newCapacity)) - newLow;
        _initNodeArray(newNode, newLow, _size, _min);
        if(capacity())
            ::operator delete (_node + _low);
        _node = newNode;
        _high = newHigh;
        _low = newLow;
    }

    /** Transforms the _node array so that it has a high value of at least
     ** newHigh; a low value of at most newLow; a size of exactly newSize;
     ** and a min of exactly newMin.  If necessary, the _node array is
     ** reallocated to guarantee that subsequently adding any id from newLow
     ** to newHigh - 1 will not require another reallocation.  In either case,
     ** _initNodeArray is invoked to add or remove node members as per the
     ** requested newSize and newMin for the Domain.
     **/
    template<typename M, typename V>
    inline void Domain<M,V>::_transform(int newHigh, int newLow, int newSize, int newMin)
    {
        if(_high == _low)
        {
            _high = newLow;
            _low  = newLow;
        }
        if(newLow < _low || newHigh > _high)
        {
            int growHigh = newHigh <= _high ? 0 : std::max(capacity(), newHigh - _high);
            int growLow = newLow >= _low ? 0 : std::max(capacity(), _low - newLow);
            newHigh = _high + growHigh;
            newLow = _low - growLow;
            int newCapacity = newHigh - newLow;
            Node* newNode = static_cast<Node*> (::operator new (sizeof(Node) * newCapacity)) - newLow;
            _initNodeArray(newNode, newLow, newSize, newMin);
            if(capacity())
                ::operator delete (_node + _low);
            _node = newNode;
            _high = newHigh;
            _low = newLow;
        }
        else
            _initNodeArray(_node, _low, newSize, newMin);
    }

    /** Initializes the node array newNode.  Like _node, newNode always
     ** points to what would be the array element with id zero even if
     ** the range of the array does not include the zero element.
     ** The first element of the array is reserved for id newLow, but only
     ** members numbered from newMin to newMin + newSize - 1 are initialized.
     ** newNode may (or may not) be the same array as _node.  In either case,
     ** elements from the _node array that are no longer used are removed from
     ** their groups and destroyed.  If newNode and _node are at different
     ** memory locations, node structures are copied from _node to newNode
     ** without disturbing group memberships (or the list order of those
     ** memberships), and the Node structures from the old _node are
     ** destroyed.  New Domain members are initialized with default M values
     ** and no group memberships.  Where possible, new Nodes are initialized
     ** starting from the lowest numbered members; and old Nodes are destroyed
     ** starting with highest numbered members.
     **
     ** _size and _min are always updated, but in no case is _low, _high
     ** or _node modified, since these values may still be required by
     ** the caller.  It is the caller's responsibility to update these
     ** values as appropriate.
     **/
    template<typename M, typename V>
    inline void Domain<M,V>::_initNodeArray(
            Node* newNode, int newLow, int newSize, int newMin)
    {
        // Throughout this method, the variable i is consistently defined as
        // an id from the Domain.

        //   1. If there are elements at the tail of _node that are
        //      numbered greater than the highest numbered elements of
        //      newNode then these elements must be removed from their
        //      groups (since they will no longer exist in the new
        //      Domain).
        //
        if(_min + _size > newMin + newSize)
        {
            int limit = std::max(_min, newMin + newSize);
            for(int i = _min + _size - 1; i >= limit; --i)
            {
                Node* n = _node + i;
                if(n->_group)
                    n->_group->_remove(n);
            }
        }

        //   2. If there are elements at the head of _node that are
        //      numbered less than the lowest numbered elements of newNode
        //      then these elements must be removed from their groups
        //      (since they will no longer exist in the new Domain).
        //
        if(_min < newMin)
        {
            for(int i = std::min(_min + _size, newMin) - 1; i >= _min; --i)
            {
                Node* n = _node + i;
                if(n->_group)
                    n->_group->_remove(n);
            }
        }

        //   3. If there are elements at the head of newNode that are
        //      numbered less lowest numbered elements of _node then these
        //      elements must be initialized.
        //
        if(newMin < _min)
        {
            int limit = std::min(newMin + newSize, _min);
            for(int i = newMin; i < limit; ++i)
                new (newNode + i) Node(i);
        }

        //   4. If newNode is a newly allocated Node array then we need to copy
        //      elements with common ids from _node to newNode.
        //
        if(newNode != _node)
        {
            int limit = std::min(_min + _size, newMin + newSize);
            for(int i = std::max(_min, newMin); i < limit; ++i)
            {
                Node* src  = _node + i;
                Node* dest = newNode + i;
                new (dest) Node(
                        src->_group,
                        !src->_group ? NULL : src->_prev == &src->_group->_end ? src->_prev : dest + (src->_prev - src),
                        !src->_group ? NULL : src->_next == &src->_group->_end ? src->_next : dest + (src->_next - src),
                        i,
                        src->_entry.member);

                // We must also offset the front and back pointers of all
                // non-empty groups.  We only want to do this once so make
                // the update when we find the front element of the group.
                //
                if(src->_group && src->_group->_end._next == src)
                {
                    src->_group->_end._next = dest;
                    src->_group->_end._prev = dest + (src->_group->_end._prev - src);
                }
            }
        }

        //   5. If there are elements at the tail of newNode that are
        //      numbered greater than the highest numbered elements of
        //      _node then these elements must be initialized.
        //
        if(newMin + newSize > _min + _size)
        {
            int limit = newMin + newSize;
            for(int i = std::max(newMin, _min + _size); i < limit; ++i)
                new (newNode + i) Node(i);
        }

        //   6. If there are elements at the tail of _node that are
        //      numbered greater than the highest numbered elements of
        //      newNode then these elements must be destroyed.
        //
        if(_min + _size > newMin + newSize)
        {
            int limit = std::max(_min, newMin + newSize);
            for(int i = _min + _size - 1; i >= limit; --i)
                (_node + i)->~Node();
        }

        //   7. If newNode is a newly allocated Node array then we need to
        //      destroy elements with common ids in _node.
        //
        if(newNode != _node)
        {
            int limit = std::max(_min, newMin);
            for(int i = std::min(_min + _size, newMin + newSize) - 1; i >= limit; --i)
                (_node + i)->~Node();
        }


        //   8. If there are elements at the head of _node that are
        //      numbered less than the lowest numbered elements of newNode
        //      then these elements must be destroyed.
        //
        if(_min < newMin)
        {
            for(int i = std::min(_min + _size, newMin) - 1; i >= _min; --i)
                (_node + i)->~Node();
        }

        //   9. Update the _size and _min settings, but don't update
        //      _node since it may still need to be deleted by the caller.
        //
        _size = newSize;
        _min = newMin;
    }

    template<typename M, typename V>
    inline const typename Domain<M,V>::Node* Domain<M,V>::_getNode(int id) const
    {
        return _node + id;
    }

    template<typename M, typename V>
    inline typename Domain<M,V>::Node* Domain<M,V>::_getNode(int id)
    {
        return _node + id;
    }

    template<typename M, typename V>
    inline void Domain<M,V>::_verifySize(const char* func, const char* file, int line, int size) const
    {
        if(size < 0)
            partitionThrow(func, file, line, "Domain size cannot be negative.");
    }

    template<typename M, typename V>
    inline void Domain<M,V>::_verifyRange(const char* func, const char* file, int line, int id) const
    {
        if(id < _min || id >= _min + _size)
            partitionThrow(func, file, line, "id is out of Domain range.");
    }

    template<typename M, typename V>
    inline void Domain<M,V>::_verifyHighLow(const char* func, const char* file, int line, int high, int low) const
    {
        if(low > high)
            partitionThrow(func, file, line, "Attempt to reserve Domain memory with low id greater than high id.");
    }

    template<typename M, typename V>
    inline void Domain<M,V>::_verifyInGroup(const char* func, const char* file, int line, const Node* n) const
    {
        if(n->_group == NULL)
            partitionThrow(func, file, line, "id is not in a Group.");
    }

    //------------------------------
    // Node implementation
    //------------------------------

    template<typename M, typename V>
    inline Domain<M,V>::Node::Node(int id)
        :   _group(NULL),
#if PARTITION_SL >= PARTITION_ST_DEBUG
            _prev(NULL),
            _next(NULL),
#endif
            _entry(id)
    {
    }

    template<typename M, typename V>
    inline Domain<M,V>::Node::Node(int id, const M& member)
        :   _group(NULL),
#if PARTITION_SL >= PARTITION_ST_DEBUG
            _prev(NULL),
            _next(NULL),
#endif
            _entry(id, member)
    {
    }

    template<typename M, typename V>
    inline Domain<M,V>::Node::Node(Group<M,V>* group, Node* prev, Node* next, int id)
        :   _group(group),
            _prev(prev),
            _next(next),
            _entry(id)
    {
    }

    template<typename M, typename V>
    inline Domain<M,V>::Node::Node(Group<M,V>* group, Node* prev, Node* next, int id, const M& member)
        :   _group(group),
            _prev(prev),
            _next(next),
            _entry(id, member)
    {
    }


    /** \class Group
     **
     ** Maintains a doubly linked list of Entry objects from a single
     ** associated Domain and may also be assigned a user-defined value
     ** of type V.
     **
     ** Read the [package summary](index.html) for introductory information
     ** about Group and its related classes.
     **
     ** An Entry may be added to a Group via add operations typical of
     ** linked lists, but each such add has the atypical side-affect of
     ** simultaneously removing the Entry from its previous Group membership
     ** (if any) all with constant computational complexity.
     **
     ** The following four terms are used consistently throughout to refer to
     ** key list posistions:
     **
     **- *beforeFront* - One position before the first entry in the list.
     **- *front* - The first entry in the list.
     **- *back* - The last entry in the list.
     **- *afterBack* - One position after the last entry in the list.
     **
     ** There is no way to copy a Group without destroying the original, so
     ** the copy constructor and operator= are both declared private (and left
     ** unimplemented).  You can, however, copy a group onto a different
     ** Domain using `Group<M,V>::Group(Domain<M,V>& domain, const Group&
     ** group)`.
     **
     ** \param M The Entry member type.
     ** \param V The Group value type.
     **/

    /** Constructs an empty Group with no Domain association and a default
     ** value.  Few operations on a Group constructed in this way are
     ** permitted until a Domain object has been associated with the Group
     ** via the setDomain() method.
     **/
    template<typename M, typename V>
    inline Group<M,V>::Group()
        :   _domain(NULL),
            _end(this, &_end, &_end, 0x80000000),
            _size(0)
    {
    }

    /** Constructs an empty Group associated with Domain `d` and a default
     ** value.
     **/
    template<typename M, typename V>
    inline Group<M,V>::Group(Domain<M,V>& d)
        :   _domain(&d),
            _end(this, &_end, &_end, 0x80000000),
            _size(0)
    {
    }

    /** Constructs an empty Group associated with Domain `d` and sets the
     ** Group's value to `v`.
     **/
    template<typename M, typename V>
    inline Group<M,V>::Group(Domain<M,V>& d, const V& v)
        :   _domain(&d),
            _end(this, &_end, &_end, 0x80000000),
            _size(0),
            _value(v)
    {
    }

    /** Constructs a Group that is a copy of `g` with the exception that
     ** its associated with Domain `d`.  Domain `d` must not be the same
     ** Domain as that associated with `g`.
     **
     ** \throw PartitionException if `g` is associated with Domain `d`,
     ** or\n if an Entry of `g` has an ID not contained in `d`.\n\n
     ** These validations are only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_GROUP_CONSTRUCTOR
     ** \endcode
     **/
    template<typename M, typename V>
    inline Group<M,V>::Group(Domain<M,V>& d, const Group& g)
        :   _domain(&d),
            _end(this, &_end, &_end, 0x80000000),
            _size(0),
            _value(g.getValue())
    {
        PARTITION_SAFETY_GROUP_CONSTRUCTOR(PARTITION_VERIFY_DIFF_DOMAIN(g));
        for(ConstIterator i = g.front(); !i.isAfterBack(); ++i)
            addBack(i->id);
    }

    /** Constructs a Group loaded with the IDs listed from `first` to `last`.
     ** The iterators first and last should point to the beginning and end
     ** of a sequence of integers that are each IDs found in Domain `d`.
     **
     ** \throw PartitionException if an `id` in the input sequence
     ** is not contained in `d`.\n\n
     ** Thsi validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_GROUP_ADD
     ** \endcode
     **/
    template<typename M, typename V> template<typename InputIterator>
    inline Group<M,V>::Group(
            Domain<M,V>& d,
            const InputIterator& first,
            const InputIterator& last)

        :   _domain(&d),
            _end(this, &_end, &_end, 0x80000000),
            _size(0)
    {
        for(InputIterator i = first; i != last; ++i)
            addBack(*i);
    }

    /** Destructs the Group, removing all Entry objects from the Group.
     **/
    template<typename M, typename V>
    inline Group<M,V>::~Group()
    {
        removeAll();
    }

    /** Returns a pointer to the Domain associated with this Group (or `NULL`
     ** if this Group has no Domain association).
     **/
    template<typename M, typename V>
    inline Domain<M,V>* Group<M,V>::getDomain()
    {
        return _domain;
    }

    /** Returns a pointer to the const Domain associated with this Group (or
     ** `NULL` if this Group has no Domain association).
     **/
    template<typename M, typename V>
    inline const Domain<M,V>* Group<M,V>::getDomain() const
    {
        return _domain;
    }

    /** Associates this Group with Domain `d`.  If this Group was not
     ** previously associated with `d`, then the Group's list is first cleared
     ** of any Entry objects from its current Domain association (if any).  If
     ** this Group was already associated with `d`, then there is no effect.
     **/
    template<typename M, typename V>
    inline void Group<M,V>::setDomain(Domain<M,V>& d)
    {
        if(_domain == &d)
            return;
        removeAll();
        _domain = &d;
    }

    /** Returns a reference to the user provided value associated with this
     ** Group.
     **/
    template<typename M, typename V>
    inline V& Group<M,V>::getValue()
    {
        return _value;
    }

    /** Returns a const reference to the user provided value associated with
     ** this Group.
     **/
    template<typename M, typename V>
    inline const V& Group<M,V>::getValue() const
    {
        return _value;
    }

    /** Sets `v` as this Group's value.
     **/
    template<typename M, typename V>
    inline void Group<M,V>::setValue(const V& v)
    {
        this->_value = v;
    }

    /** Returns the number of Entry objects in this Group's list.
     **/
    template<typename M, typename V>
    inline int Group<M,V>::size() const
    {
        return _size;
    }

    /** Returns `true` if this Group contains the Entry with the given `id`,
     ** and false otherwise.
     **
     ** Computational Complexity:  Constant.
     **
     ** \throw PartitionException if this Group has no Domain association; or\n
     ** if `id` is not a member of the associated Domain.\n\n
     ** These validations are only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_GROUP_ACCESSOR
     ** \endcode
     **/
    template<typename M, typename V>
    inline bool Group<M,V>::contains(int id) const
    {
        PARTITION_SAFETY_GROUP_ACCESSOR(PARTITION_VERIFY_DOMAIN());
        PARTITION_SAFETY_GROUP_ACCESSOR(_domain->PARTITION_VERIFY_RANGE(id));
        return _domain->_getNode(id)->_group == this;
    }

    /** Returns a reference to the Entry at the front of the Group list
     ** while leaving the list unmodified.
     **
     ** \throw PartitionException if the Group is empty.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_GROUP_ACCESSOR
     ** \endcode
     **/
    template<typename M, typename V>
    inline typename Domain<M,V>::Entry& Group<M,V>::peekFront()
    {
        PARTITION_SAFETY_GROUP_ACCESSOR(PARTITION_VERIFY_NOT_EMPTY());
        return _end._next->_entry;
    }

    /** Returns a const reference to the Entry at the front of the
     ** Group list while leaving the list unmodified.
     **
     ** \throw PartitionException if the Group is empty.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_GROUP_ACCESSOR
     ** \endcode
     **/
    template<typename M, typename V>
    inline const typename Domain<M,V>::Entry& Group<M,V>::peekFront() const
    {
        PARTITION_SAFETY_GROUP_ACCESSOR(PARTITION_VERIFY_NOT_EMPTY());
        return _end._next->_entry;
    }

    /** Returns a reference to the Entry at the back of the Group list
     ** while leaving the list unmodified.
     **
     ** \throw PartitionException if the Group is empty.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_GROUP_ACCESSOR
     ** \endcode
     **/
    template<typename M, typename V>
    inline typename Domain<M,V>::Entry& Group<M,V>::peekBack()
    {
        PARTITION_SAFETY_GROUP_ACCESSOR(PARTITION_VERIFY_NOT_EMPTY());
        return _end._prev->_entry;
    }

    /** Returns a const reference to the Entry at the back of the Group
     ** list while leaving the list unmodified.
     **
     ** \throw PartitionException if the Group is empty.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_GROUP_ACCESSOR
     ** \endcode
     **/
    template<typename M, typename V>
    inline const typename Domain<M,V>::Entry& Group<M,V>::peekBack() const
    {
        PARTITION_SAFETY_GROUP_ACCESSOR(PARTITION_VERIFY_NOT_EMPTY());
        return _end._prev->_entry;
    }

    /** Adds the Entry with the given `id` from the Group's associated Domain
     ** to the front of this Group's list.  If the Entry was previously a
     ** member of a Group, it is first removed from that Group.
     **
     ** Computational Complexity:  Constant.
     **
     ** \throw PartitionException if this Group has no Domain association; or\n
     ** if `id` is not a member of the associated Domain.\n\n
     ** These validations are only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_GROUP_ADD
     ** \endcode
     **/
    template<typename M, typename V>
    inline void Group<M,V>::addFront(int id)
    {
        PARTITION_SAFETY_GROUP_ADD(PARTITION_VERIFY_DOMAIN());
        PARTITION_SAFETY_GROUP_ADD(_domain->PARTITION_VERIFY_RANGE(id));
        _addAfter(&_end, _domain->_getNode(id));
    }

    /** Adds the Entry with the given `id` from the Group's associated Domain
     ** to the back of this Group's list.  If the Entry was previously a member
     ** of a Group, it is first removed from that Group.
     **
     ** Computational Complexity:  Constant.
     **
     ** \throw PartitionException if this Group has no Domain association; or\n
     ** if `id` is not a member of the associated Domain.\n\n
     ** These validations are only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_GROUP_ADD
     ** \endcode
     **/
    template<typename M, typename V>
    inline void Group<M,V>::addBack(int id)
    {
        PARTITION_SAFETY_GROUP_ADD(PARTITION_VERIFY_DOMAIN());
        PARTITION_SAFETY_GROUP_ADD(_domain->PARTITION_VERIFY_RANGE(id));
        _addAfter(_end._prev, _domain->_getNode(id));
    }

    /** Moves all Entry objects from Group `g` to the front of this Group's
     ** list.
     **
     ** Computational Complexity:  Linear in the size of `g`.
     **
     ** \throw PartitionException if Group `g` is not associated with the same
     ** Domain as this Group.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_GROUP_ADD
     ** \endcode
     **/
    template<typename M, typename V>
    inline void Group<M,V>::addFront(Group& g)
    {
        if(this == &g)
            return;

        PARTITION_SAFETY_GROUP_ADD(PARTITION_VERIFY_DOMAIN());
        PARTITION_SAFETY_GROUP_ADD(PARTITION_VERIFY_SAME_DOMAIN(g));

        if(g._size == 0)
            return;

        Iterator i(this, _end._next);

        Node * na = &_end;
        Node * nb = g._end._next;
        Node * nc = g._end._prev;
        Node * nd = _end._next;

        na->_next = nb;
        nb->_prev = na;
        nc->_next = nd;
        nd->_prev = nc;

        g._end._next = &g._end;
        g._end._prev = &g._end;
        _size += g._size;
        g._size = 0;

        // Still need to update all the Group pointers.
        //
        while(!(--i).isBeforeFront())
            i._node->_group = this;
    }

    /** Moves all Entry objects from Group g to the back of this Group's
     ** list.
     **
     ** Computational Complexity:  Linear in the size of `g`.
     **
     ** \throw PartitionException if Group `g` is not associated with the same
     ** Domain as this Group.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_GROUP_ADD
     ** \endcode
     **/
    template<typename M, typename V>
    inline void Group<M,V>::addBack(Group& g)
    {
        if(g._size == 0)
            return;

        if(this == &g)
            return;

        PARTITION_SAFETY_GROUP_ADD(PARTITION_VERIFY_DOMAIN());
        PARTITION_SAFETY_GROUP_ADD(PARTITION_VERIFY_SAME_DOMAIN(g));

        Iterator i(this, _end._prev);

        Node * na = _end._prev;
        Node * nb = g._end._next;
        Node * nc = g._end._prev;
        Node * nd = &_end;

        na->_next = nb;
        nb->_prev = na;
        nc->_next = nd;
        nd->_prev = nc;

        g._end._next = &g._end;
        g._end._prev = &g._end;

        _size += g._size;
        g._size = 0;

        // Still need to update all the Group pointers.
        //
        while(!(++i).isAfterBack())
            i._node->_group = this;
    }

    /** Adds all Entry objects from this Group's Domain to this Group's
     ** list (simultaneously removing them from the Groups with which they
     ** were previously a member).  Entry objects are listed in ascending ID
     ** order.
     **
     ** Computational Complexity:  Linear in the size of this Group's Domain.
     **
     ** \throw PartitionException if this Group has no Domain association\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_GROUP_ADD
     ** \endcode
     **/
    template<typename M, typename V>
    inline void Group<M,V>::addAll()
    {
        PARTITION_SAFETY_GROUP_ADD(PARTITION_VERIFY_DOMAIN());
        int begin = _domain->min();
        int end = _domain->min() + _domain->size();
        for(int id = begin; id < end; ++id)
            addBack(id);
    }

    /** Removes the Entry at the front of this Group's list and returns a
     ** reference to it.
     **
     ** Computational Complexity:  Constant.
     **
     ** \throw PartitionException if this Group is empty.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_GROUP_REMOVE
     ** \endcode
     **/
    template<typename M, typename V>
    inline typename Domain<M,V>::Entry& Group<M,V>::removeFront()
    {
        PARTITION_SAFETY_GROUP_REMOVE(PARTITION_VERIFY_NOT_EMPTY());
        return _remove(_end._next)->_entry;
    }

    /** Removes the Entry at the back of this Group's list and returns a
     ** reference to it.
     **
     ** Computational Complexity:  Constant.
     **
     ** \throw PartitionException if this Group is empty.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_GROUP_REMOVE
     ** \endcode
     **/
    template<typename M, typename V>
    inline typename Domain<M,V>::Entry& Group<M,V>::removeBack()
    {
        PARTITION_SAFETY_GROUP_REMOVE(PARTITION_VERIFY_NOT_EMPTY());
        return _remove(_end._prev)->_entry;
    }

    /** Removes from this Group's list the Entry with the given `id` and returns
     ** reference to it.
     **
     ** Computational Complexity:  Constant.
     **
     ** \throw PartitionException if this Group has no Domain association; or\n
     ** if this Group does not contain the Entry with the given `id`.\n\n
     ** These validations are only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_GROUP_REMOVE
     ** \endcode
     **/
    template<typename M, typename V>
    inline typename Domain<M,V>::Entry& Group<M,V>::remove(int id)
    {
        PARTITION_SAFETY_GROUP_REMOVE(PARTITION_VERIFY_DOMAIN());
        PARTITION_SAFETY_GROUP_REMOVE(_domain->PARTITION_VERIFY_RANGE(id));
        Node* n = _domain->_getNode(id);
        PARTITION_SAFETY_GROUP_REMOVE(PARTITION_VERIFY_IN_GROUP(n));
        _remove(n);
        return n->_entry;
    }

    /** Removes all Entry objects from this Group's list.
     **
     ** Computational Complexity:  Linear in the size of this Group.
     **/
    template<typename M, typename V>
    inline void Group<M,V>::removeAll()
    {
        // Could just write:
        //
        //   while(size())
        //     remove_front();
        //
        // But this should be considerably faster:
        //
        Node* n = _end._next;
        while(n != &_end)
        {
            Node* tmp = n;
            n = n->_next;
            tmp->_group = NULL;
#if PARTITION_SL >= PARTITION_ST_DEBUG
            tmp->_prev = NULL;
            tmp->_next = NULL;
#endif
        }

        _end._next = &_end;
        _end._prev = &_end;
        _size      = 0;
    }

    /** Returns an Iterator pointing to the first Entry of this Group's
     ** list.  If this Group is empty, the returned Iterator will point to the
     ** afterBack list position.
     **/
    template<typename M, typename V>
    inline typename Group<M,V>::Iterator Group<M,V>::front()
    {
        return Iterator(this, _end._next);
    }

    /** Returns a ConstIterator pointing to the first Entry of this Group's
     ** list.  If this Group is empty, the returned ConstIterator will point
     ** to the afterBack list position.
     **/
    template<typename M, typename V>
    inline typename Group<M,V>::ConstIterator Group<M,V>::front() const
    {
        return ConstIterator(this, _end._next);
    }

    /** Returns an Iterator pointing to the last Entry of this Group's list.
     ** If this Group is empty, the returned Iterator will point to the
     ** beforeFront list position.
     **/
    template<typename M, typename V>
    inline typename Group<M,V>::Iterator Group<M,V>::back()
    {
        return Iterator(this, _end._prev);
    }

    /** Returns a ConstIterator pointing to the last Entry of this Group's
     ** list.  If this Group is empty, the returned ConstIterator will point
     ** to the beforeFront list position.
     **/
    template<typename M, typename V>
    inline typename Group<M,V>::ConstIterator Group<M,V>::back() const
    {
        return ConstIterator(this, _end._prev);
    }

    /** Returns an Iterator pointing to the position before the first
     ** Entry in this Group's list.
     **/
    template<typename M, typename V>
    inline typename Group<M,V>::Iterator Group<M,V>::beforeFront()
    {
        return Iterator(this, &_end);
    }

    /** Returns a ConstIterator pointing to the position before the first
     ** Entry in this Group's list.
     **/
    template<typename M, typename V>
    inline typename Group<M,V>::ConstIterator Group<M,V>::beforeFront() const
    {
        return ConstIterator(this, &_end);
    }

    /** Returns an Iterator pointing to the position after the last
     ** Entry in this Group's list.
     **/
    template<typename M, typename V>
    inline typename Group<M,V>::Iterator Group<M,V>::afterBack()
    {
        return Iterator(this, &_end);
    }

    /** Returns a ConstIterator pointing to the position after the last
     ** Entry in this Group's list.
     **/
    template<typename M, typename V>
    inline typename Group<M,V>::ConstIterator Group<M,V>::afterBack() const
    {
        return ConstIterator(this, &_end);
    }

    /** Equivalent to front().
     **/
    template<typename M, typename V>
    inline typename Group<M,V>::Iterator Group<M,V>::begin()
    {
        return front();
    }

    /** Equivalent to front() const.
     **/
    template<typename M, typename V>
    inline typename Group<M,V>::ConstIterator Group<M,V>::begin() const
    {
        return front();
    }

    /** Equivalent to afterBack().
     **/
    template<typename M, typename V>
    inline typename Group<M,V>::Iterator Group<M,V>::end()
    {
        return afterBack();
    }

    /** Equivalent to afterBack() const.
     **/
    template<typename M, typename V>
    inline typename Group<M,V>::ConstIterator Group<M,V>::end() const
    {
        return afterBack();
    }

    /** Returns an Iterator pointing to the Entry from this list with the
     ** given `id`, or pointing to the afterBack position if this Group does
     ** not contain the Entry with the given `id`.
     **
     ** **PLEASE NOTE THE EXCEPTIONAL CONDITION BELOW.**
     **
     ** \throw PartitionException if this Group has no Domain association; or\n
     ** if `id` is not a member of the associated Domain.\n\n
     ** These validations are only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_GROUP_ACCESSOR
     ** \endcode
     **/
    template<typename M, typename V>
    inline typename Group<M,V>::Iterator Group<M,V>::find(int id)
    {
        PARTITION_SAFETY_GROUP_ACCESSOR(PARTITION_VERIFY_DOMAIN());
        PARTITION_SAFETY_GROUP_ACCESSOR(_domain->PARTITION_VERIFY_RANGE(id));
        Node* n = _domain->_getNode(id);
        return Iterator(this, n->_group == this ? n : &_end);
    }

    /** Returns an ConstIterator pointing to the Entry from this list
     ** with the given `id`, or pointing to the afterBack position if this
     ** Group does not contain the Entry with the given `id`.
     **
     ** **PLEASE NOTE THE EXCEPTIONAL CONDITION BELOW.**
     **
     ** \throw PartitionException if this Group has no Domain association; or\n
     ** if `id` is not a member of the associated Domain.\n\n
     ** These validations are only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_GROUP_ACCESSOR
     ** \endcode
     **/
    template<typename M, typename V>
    inline typename Group<M,V>::ConstIterator Group<M,V>::find(int id) const
    {
        PARTITION_SAFETY_GROUP_ACCESSOR(PARTITION_VERIFY_DOMAIN());
        PARTITION_SAFETY_GROUP_ACCESSOR(_domain->PARTITION_VERIFY_RANGE(id));
        const Node* n = _domain->_getNode(id);
        return ConstIterator(this, n->_group == this ? n : &_end);
    }

    template<typename M, typename V>
    inline void Group<M,V>::_addAfter(Node* n1, Node* n2)
    {
        Node* n3 = n1->_next;
        if(n2 == n1 || n2 == n3)
            return;
        if(n2->_group)
            n2->_group->_remove(n2);
        n1->_next = n2;
        n2->_prev = n1;
        n2->_next = n3;
        n3->_prev = n2;
        n2->_group = this;
        ++_size;
    }

    template<typename M, typename V>
    inline typename Domain<M,V>::Node* Group<M,V>::_remove(Node* node)
    {
        node->_prev->_next = node->_next;
        node->_next->_prev = node->_prev;
        --_size;
        node->_group = NULL;
#if PARTITION_SL >= PARTITION_ST_DEBUG
        node->_prev = NULL;
        node->_next = NULL;
#endif
        return node;
    }

    template<typename M, typename V>
    inline void Group<M,V>::_verifyDomain(const char* func, const char* file, int line) const
    {
        if(_domain == NULL)
            partitionThrow(func, file, line, "Group is not associated with a Domain.");
    }

    template<typename M, typename V>
    inline void Group<M,V>::_verifySameDomain(const char* func, const char* file, int line, const Group<M,V>& g) const
    {
        if(_domain != g._domain)
            partitionThrow(func, file, line, "given Group is associated with a different Domain than this Group.");
    }

    template<typename M, typename V>
    inline void Group<M,V>::_verifyDiffDomain(const char* func, const char* file, int line, const Group<M,V>& g) const
    {
        if(_domain == g._domain)
            partitionThrow(func, file, line, "given Group is associated with the same Domain as this Group.");
    }

    template<typename M, typename V>
    inline void Group<M,V>::_verifyNotEmpty(const char* func, const char* file, int line) const
    {
        if(!_size)
            partitionThrow(func, file, line, "Group is empty.");
    }

    template<typename M, typename V>
    inline void Group<M,V>::_verifyInGroup(const char* func, const char* file, int line, const Node* n) const
    {
        if(n->_group != this)
            partitionThrow(func, file, line, "id is not in this Group.");
    }

    /** \class Group::Iterator
     **
     ** A bidirectional iterator for traversing the Entry objects of a Group.
     **
     ** **Notes on Iterator invalidation:**
     **
     ** An Iterator is invalidated if either of the following occur:
     **
     **- The Entry the Iterator points to is removed from its Group.
     **- The underlying Domain incurs a memory reallocation event.
     **
     ** The behavior of an invalidated Iterator is undefined so one must take
     ** care to avoid unintentional iterator invalidation.  Remember that an
     ** operation that adds an Entry to a Group also removes that same
     ** Entry from its previous Group; so all operations that affect Group
     ** membership can invalidate Iterators.  Also the Domain's resize methods
     ** can be used to eliminate Entry's from the Domain entirely (and from
     ** their Groups).
     **
     ** For more information on the circumstances of Domain memory
     ** reallocation events see Domain.
     **/

    /** Constructs a Iterator with a `NULL` Group association.  Few operations
     ** on an Iterator constructed in this Way are permitted until the
     ** Iterator is assigned a value.
     **/
    template<typename M, typename V>
    inline Group<M,V>::Iterator::Iterator()
        :   _group(NULL),
            _node(NULL)
    {
    }

    /** Copy constructor.
     **/
    template<typename M, typename V>
    inline Group<M,V>::Iterator::Iterator(const Iterator& i)
        :   _group(i._group),
            _node(i._node)
    {
    }

    /** Copy assignment.
     **/
    template<typename M, typename V>
    inline typename Group<M,V>::Iterator& Group<M,V>::Iterator::operator=(const Group<M,V>::Iterator& i)
    {
        _group = i._group;
        _node = i._node;
        return *this;
    }

    /** Increments the iterator to the next list position and returns a
     ** reference to itself.
     **
     ** \throw PartitionException if this Iterator has no Group
     ** association.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_ITERATOR_INC
     ** \endcode
     **/
    template<typename M, typename V>
    inline typename Group<M,V>::Iterator& Group<M,V>::Iterator::operator++()
    {
        PARTITION_SAFETY_ITERATOR_DEREF(PARTITION_VERIFY_GROUP());
        _node = _node->_next;
        return *this;
    }

    /** Decrements the iterator to the previous list position and
     ** returns a reference to itself.
     **
     ** \throw PartitionException if this Iterator has no Group
     ** association.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_ITERATOR_INC
     ** \endcode
     **/
    template<typename M, typename V>
    inline typename Group<M,V>::Iterator& Group<M,V>::Iterator::operator--()
    {
        PARTITION_SAFETY_ITERATOR_DEREF(PARTITION_VERIFY_GROUP());
        _node = _node->_prev;
        return *this;
    }

    /** Increments the iterator to the next list position and returns a
     ** copy of the value of this iterator before the increment.
     **
     ** \throw PartitionException if this Iterator has no Group
     ** association.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_ITERATOR_INC
     ** \endcode
     **/
    template<typename M, typename V>
    inline typename Group<M,V>::Iterator Group<M,V>::Iterator::operator++(int)
    {
        PARTITION_SAFETY_ITERATOR_DEREF(PARTITION_VERIFY_GROUP());
        Iterator ret = *this;
        _node = _node->_next;
        return ret;
    }

    /** Decrements the iterator to the previous list position and
     ** returns a copy of the value of this iterator before the decrement.
     **
     ** \throw PartitionException if this Iterator has no Group
     ** association.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_ITERATOR_INC
     ** \endcode
     **/
    template<typename M, typename V>
    inline typename Group<M,V>::Iterator Group<M,V>::Iterator::operator--(int)
    {
        PARTITION_SAFETY_ITERATOR_DEREF(PARTITION_VERIFY_GROUP());
        Iterator ret = *this;
        _node = _node->_prev;
        return ret;
    }

    /** Returns `true` if this iterator is pointing one logical position
     ** before the first Entry in the list, and `false` if it is pointing
     ** to any list Entry.
     **
     ** \throw PartitionException if this Iterator has no Group
     ** association.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_ITERATOR_DEREF
     ** \endcode
     **/
    template<typename M, typename V>
    inline bool Group<M,V>::Iterator::isBeforeFront() const
    {
        PARTITION_SAFETY_ITERATOR_DEREF(PARTITION_VERIFY_GROUP());
        return _node == &_group->_end;
    }

    /** Returns `true` if this iterator is pointing one logical position
     ** after the last Entry in the list, and `false` if it is pointing
     ** to any list Entry.
     **
     ** \throw PartitionException if this Iterator has no Group
     ** association.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_ITERATOR_DEREF
     ** \endcode
     **/
    template<typename M, typename V>
    inline bool Group<M,V>::Iterator::isAfterBack() const
    {
        PARTITION_SAFETY_ITERATOR_DEREF(PARTITION_VERIFY_GROUP());
        return _node == &_group->_end;
    }

    /** Returns `true` if this iterator points to the first Entry of
     ** a non-empty list, or to the afterBack position of an empty
     ** list; and `false` otherwise.
     **
     ** \throw PartitionException if this Iterator has no Group
     ** association.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_ITERATOR_DEREF
     ** \endcode
     **/
    template<typename M, typename V>
    inline bool Group<M,V>::Iterator::isAtFront() const
    {
        PARTITION_SAFETY_ITERATOR_DEREF(PARTITION_VERIFY_GROUP());
        return _node == _group->_end._next;
    }

    /** Returns `true` if this iterator points to the last Entry of
     ** a non-empty list, or to the beforeFront position of an empty
     ** list; and `false` otherwise.
     **
     ** \throw PartitionException if this Iterator has no Group
     ** association.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_ITERATOR_DEREF
     ** \endcode
     **/
    template<typename M, typename V>
    inline bool Group<M,V>::Iterator::isAtBack() const
    {
        PARTITION_SAFETY_ITERATOR_DEREF(PARTITION_VERIFY_GROUP());
        return _node == _group->_end._prev;
    }

    /** Returns a reference to the Entry pointed to by this iterator.
     **
     ** \throw PartitionException if this Iterator `isBeforeFront()` or
     ** `isAfterBack()`.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_ITERATOR_DEREF
     ** \endcode
     **/
    template<typename M, typename V>
    inline typename Domain<M,V>::Entry& Group<M,V>::Iterator::operator*() const
    {
        PARTITION_SAFETY_ITERATOR_DEREF(PARTITION_VERIFY_GROUP());
        PARTITION_SAFETY_ITERATOR_DEREF(PARTITION_VERIFY_NOT_AT_END());
        return _node->_entry;
    }

    /** Returns a pointer to the Entry pointed to by this iterator.
     **
     ** \throw PartitionException if this Iterator `isBeforeFront()` or
     ** `isAfterBack()`.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_ITERATOR_DEREF
     ** \endcode
     **/
    template<typename M, typename V>
    inline typename Domain<M,V>::Entry* Group<M,V>::Iterator::operator->() const
    {
        PARTITION_SAFETY_ITERATOR_DEREF(PARTITION_VERIFY_GROUP());
        PARTITION_SAFETY_ITERATOR_DEREF(PARTITION_VERIFY_NOT_AT_END());
        return &_node->_entry;
    }

    /** Returns a pointer to the Group associated with this Iterator.
     **/
    template<typename M, typename V>
    inline Group<M,V>* Group<M,V>::Iterator::getContainer() const
    {
        return _group;
    }

    /** Returns `true` if this Iterator and `i` point to the same Entry
     ** in the same Group list, and `false` otherwise.
     **
     ** \throw PartitionException if this Iterator and Iterator i are not
     ** associated with the same Group.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_ITERATOR_COMPARE
     ** \endcode
     **/
    template<typename M, typename V>
    inline bool Group<M,V>::Iterator::operator==(const Iterator& i) const
    {
        PARTITION_SAFETY_ITERATOR_COMPARE(PARTITION_VERIFY_COMPARE(i));
        return i._node == _node;
    }

    /** Returns `true` if this Iterator and `i` point to the same Entry
     ** in the same Group list, and `false` otherwise.
     **
     ** \throw PartitionException if this Iterator and Iterator i are not
     ** associated with the same Group.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_ITERATOR_COMPARE
     ** \endcode
     **/
    template<typename M, typename V>
    inline bool Group<M,V>::Iterator::operator==(const ConstIterator& i) const
    {
        PARTITION_SAFETY_ITERATOR_COMPARE(PARTITION_VERIFY_COMPARE(i));
        return i._node == _node;
    }

    /** Returns `false` if this Iterator and `i` point to the same Entry
     ** in the same Group list and `true` otherwise.
     **
     ** \throw PartitionException if this Iterator and Iterator i are not
     ** associated with the same Group.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_ITERATOR_COMPARE
     ** \endcode
     **/
    template<typename M, typename V>
    inline bool Group<M,V>::Iterator::operator!=(const Iterator& i) const
    {
        PARTITION_SAFETY_ITERATOR_COMPARE(PARTITION_VERIFY_COMPARE(i));
        return i._node != _node;
    }

    /** Returns `false` if this Iterator and `i` point to the same Entry
     ** in the same Group list and `true` otherwise.
     **
     ** \throw PartitionException if this Iterator and Iterator i are not
     ** associated with the same Group.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_ITERATOR_COMPARE
     ** \endcode
     **/
    template<typename M, typename V>
    inline bool Group<M,V>::Iterator::operator!=(const ConstIterator& i) const
    {
        PARTITION_SAFETY_ITERATOR_COMPARE(PARTITION_VERIFY_COMPARE(i));
        return i._node != _node;
    }

    /** Removes the Entry with the given `id` from the Group with which it
     ** was previously a member (if any), and adds it into this Iterator's
     ** Group immediately before the position pointed to by this Iterator.
     **
     ** Computational Complexity:  Constant
     **
     ** \throw PartitionException if this Iterator is not associated with a Group; or\n
     ** if this Iterator's Group is not associated with a Domain; or\n
     ** if `id` is not a member of the associated Domain.\n\n
     ** These validations are only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_GROUP_ADD
     ** \endcode
     **/
    template<typename M, typename V>
    inline void Group<M,V>::Iterator::addBefore(int id) const
    {
        PARTITION_SAFETY_GROUP_ADD(PARTITION_VERIFY_GROUP());
        PARTITION_SAFETY_GROUP_ADD(_group->PARTITION_VERIFY_DOMAIN());
        PARTITION_SAFETY_GROUP_ADD(_group->_domain->PARTITION_VERIFY_RANGE(id));
        _group->_addAfter(_node->_prev, _group->_domain->_getNode(id));
    }

    /** Removes the Entry with the given `id` from the Group with which it
     ** was previously a member (if any), and adds it into this Iterator's
     ** Group immediately after the position pointed to by this Iterator.
     **
     ** Computational Complexity:  Constant
     **
     ** \throw PartitionException if this Iterator is not associated with a Group; or\n
     ** if this Iterator's Group is not associated with a Domain; or\n
     ** if `id` is not a member of the associated Domain.\n\n
     ** These validations are only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_GROUP_ADD
     ** \endcode
     **/
    template<typename M, typename V>
    inline void Group<M,V>::Iterator::addAfter(int id) const
    {
        PARTITION_SAFETY_GROUP_ADD(PARTITION_VERIFY_GROUP());
        PARTITION_SAFETY_GROUP_ADD(_group->PARTITION_VERIFY_DOMAIN());
        PARTITION_SAFETY_GROUP_ADD(_group->_domain->PARTITION_VERIFY_RANGE(id));
        _group->_addAfter(_node, _group->_domain->_getNode(id));
    }

    /** Removes and returns a reference to the Entry pointed to by this
     ** Iterator from its Group's list, and updates this Iterator to point to
     ** the next Entry in the list.
     **
     ** Computational Complexity:  Constant.
     **
     ** \throw PartitionException if Iterator `i` `isBeforeFront()` or
     ** `isAfterBack()`.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_GROUP_REMOVE
     ** \endcode
     **/
    template<typename M, typename V>
    inline typename Domain<M,V>::Entry& Group<M,V>::Iterator::removeAndInc()
    {
        PARTITION_SAFETY_GROUP_REMOVE(PARTITION_VERIFY_GROUP());
        PARTITION_SAFETY_GROUP_REMOVE(PARTITION_VERIFY_NOT_AT_END());
        Node* n = _node;
        _node = _node->_next;
        _group->_remove(n);
        return n->_entry;
    }

    /** Removes and returns a reference to the Entry pointed to by this
     ** Iterator from this Group's list, and updates this Iterator to point to
     ** the previous Entry in the list.
     **
     ** Computational Complexity:  Constant.
     **
     ** \throw PartitionException if Iterator `i` `isBeforeFront()` or
     ** `isAfterBack()`.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_GROUP_REMOVE
     ** \endcode
     **/
    template<typename M, typename V>
    inline typename Domain<M,V>::Entry& Group<M,V>::Iterator::removeAndDec()
    {
        PARTITION_SAFETY_GROUP_REMOVE(PARTITION_VERIFY_GROUP());
        PARTITION_SAFETY_GROUP_REMOVE(PARTITION_VERIFY_NOT_AT_END());
        Node* n = _node;
        _node = _node->_prev;
        _group->_remove(n);
        return n->_entry;
    }

    template<typename M, typename V>
    inline Group<M,V>::Iterator::Iterator(Group* group, Node* node)
        :   _group(group),
            _node(node)
    {
    }

    template<typename M, typename V>
    inline void Group<M,V>::Iterator::_verifyGroup(const char* func, const char* file, int line) const
    {
        if(!_group)
            partitionThrow(func, file, line, "Iterator is not associated with a Group.");
    }

    template<typename M, typename V>
    inline void Group<M,V>::Iterator::_verifyNotAtEnd(const char* func, const char* file, int line) const
    {
        if(_node == &_group->_end)
            partitionThrow(func, file, line, "Iterator is before front or after back.");
    }

    template<typename M, typename V>
    inline void Group<M,V>::Iterator::_verifyCompare(const char* func, const char* file, int line, const Iterator& i) const
    {
        if(this->_group != i._group)
            partitionThrow(func, file, line, "Attempt to compare iterators from different groups.");
    }

    template<typename M, typename V>
    inline void Group<M,V>::Iterator::_verifyCompare(const char* func, const char* file, int line, const ConstIterator& i) const
    {
        if(this->_group != i._group)
            partitionThrow(func, file, line, "Attempt to compare iterators from different groups.");
    }

    /** \class Group::ConstIterator
     **
     ** A bidirectional iterator for traversing the Entry
     ** objects of a const Group.
     **
     ** \sa Iterator (Notes on Iterator invalidation)
     **/

    /** Constructs a ConstIterator with a `NULL` Group association.  Few
     ** operations on a ConstIterator constructed in this Way are permitted
     ** until the ConstIterator is assigned a value.
     **/
    template<typename M, typename V>
    inline Group<M,V>::ConstIterator::ConstIterator()
        :   _group(NULL),
            _node(NULL)
    {
    }

    /** Constructs a ConstIterator from an Iterator.
     **/
    template<typename M, typename V>
    inline Group<M,V>::ConstIterator::ConstIterator(const Iterator& i)
        :   _group(i._group),
            _node(i._node)
    {
    }

    /** Copy constructor.
     **/
    template<typename M, typename V>
    inline Group<M,V>::ConstIterator::ConstIterator(const ConstIterator& i)
        :   _group(i._group),
            _node(i._node)
    {
    }

    /** Assigns the value of an Iterator to a ConstIterator.
     **/
    template<typename M, typename V>
    inline typename Group<M,V>::ConstIterator& Group<M,V>::ConstIterator::operator=(const Iterator& i)
    {
        _group = i._group;
        _node = i._node;
        return *this;
    }

    /** Copy assignment.
     **/
    template<typename M, typename V>
    inline typename Group<M,V>::ConstIterator& Group<M,V>::ConstIterator::operator=(const ConstIterator& i)
    {
        _group = i._group;
        _node = i._node;
        return *this;
    }

    /** Increments the iterator to the next list position and returns a
     ** reference to itself.
     **
     ** \throw PartitionException if this Iterator has no Group
     ** association.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_ITERATOR_INC
     ** \endcode
     **/
    template<typename M, typename V>
    inline typename Group<M,V>::ConstIterator& Group<M,V>::ConstIterator::operator++()
    {
        PARTITION_SAFETY_ITERATOR_DEREF(PARTITION_VERIFY_GROUP());
        _node = _node->_next;
        return *this;
    }

    /** Decrements the iterator to the previous list position and
     ** returns a reference to itself.
     **
     ** \throw PartitionException if this Iterator has no Group
     ** association.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_ITERATOR_INC
     ** \endcode
     **/
    template<typename M, typename V>
    inline typename Group<M,V>::ConstIterator& Group<M,V>::ConstIterator::operator--()
    {
        PARTITION_SAFETY_ITERATOR_DEREF(PARTITION_VERIFY_GROUP());
        _node = _node->_prev;
        return *this;
    }

    /** Increments the iterator to the next list position and returns a
     ** copy of the value of this iterator before the increment.
     **
     ** \throw PartitionException if this Iterator has no Group
     ** association.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_ITERATOR_INC
     ** \endcode
     **/
    template<typename M, typename V>
    inline typename Group<M,V>::ConstIterator Group<M,V>::ConstIterator::operator++(int)
    {
        PARTITION_SAFETY_ITERATOR_DEREF(PARTITION_VERIFY_GROUP());
        ConstIterator ret = *this;
        _node = _node->_next;
        return ret;
    }

    /** Decrements the iterator to the previous list position and
     ** returns a copy of the value of this iterator before the decrement.
     **
     ** \throw PartitionException if this Iterator has no Group
     ** association.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_ITERATOR_INC
     ** \endcode
     **/
    template<typename M, typename V>
    inline typename Group<M,V>::ConstIterator Group<M,V>::ConstIterator::operator--(int)
    {
        PARTITION_SAFETY_ITERATOR_DEREF(PARTITION_VERIFY_GROUP());
        ConstIterator ret = *this;
        _node = _node->_prev;
        return ret;
    }

    /** Returns `true` if this iterator is pointing one logical position
     ** before the first Entry in the list, and `false` otherwise.
     **
     ** \throw PartitionException if this Iterator has no Group
     ** association.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_ITERATOR_DEREF
     ** \endcode
     **/
    template<typename M, typename V>
    inline bool Group<M,V>::ConstIterator::isBeforeFront() const
    {
        PARTITION_SAFETY_ITERATOR_DEREF(PARTITION_VERIFY_GROUP());
        return _node == &_group->_end;
    }

    /** Returns `true` if this iterator is pointing one logical position
     ** after the last Entry in the list, and `false` otherwise.
     **
     ** \throw PartitionException if this Iterator has no Group
     ** association.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_ITERATOR_DEREF
     ** \endcode
     **/
    template<typename M, typename V>
    inline bool Group<M,V>::ConstIterator::isAfterBack() const
    {
        PARTITION_SAFETY_ITERATOR_DEREF(PARTITION_VERIFY_GROUP());
        return _node == &_group->_end;
    }

    /** Returns `true` if this iterator is pointing to the first Entry in
     ** the list, and `false` otherwise.
     **
     ** \throw PartitionException if this Iterator has no Group
     ** association.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_ITERATOR_DEREF
     ** \endcode
     **/
    template<typename M, typename V>
    inline bool Group<M,V>::ConstIterator::isAtFront() const
    {
        PARTITION_SAFETY_ITERATOR_DEREF(PARTITION_VERIFY_GROUP());
        return _node == _group->_end._next;
    }

    /** Returns `true` if this iterator is pointing to the last Entry in
     ** the list, and `false` otherwise.
     **
     ** \throw PartitionException if this Iterator has no Group
     ** association.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_ITERATOR_DEREF
     ** \endcode
     **/
    template<typename M, typename V>
    inline bool Group<M,V>::ConstIterator::isAtBack() const
    {
        PARTITION_SAFETY_ITERATOR_DEREF(PARTITION_VERIFY_GROUP());
        return _node == _group->_end._prev;
    }

    /** Returns a reference to the const Entry pointed to by this iterator.
     **
     ** \throw PartitionException if this Iterator isBeforeFront() or
     ** isAfterBack().\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_ITERATOR_DEREF
     ** \endcode
     **/
    template<typename M, typename V>
    inline const typename Domain<M,V>::Entry& Group<M,V>::ConstIterator::operator*() const
    {
        PARTITION_SAFETY_ITERATOR_DEREF(PARTITION_VERIFY_GROUP());
        PARTITION_SAFETY_ITERATOR_DEREF(PARTITION_VERIFY_NOT_AT_END());
        return _node->_entry;
    }

    /** Returns a pointer to the const Entry pointed to by this iterator.
     **
     ** \throw PartitionException if this Iterator isBeforeFront() or
     ** isAfterBack().\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_ITERATOR_DEREF
     ** \endcode
     **/
    template<typename M, typename V>
    inline const typename Domain<M,V>::Entry* Group<M,V>::ConstIterator::operator->() const
    {
        PARTITION_SAFETY_ITERATOR_DEREF(PARTITION_VERIFY_GROUP());
        PARTITION_SAFETY_ITERATOR_DEREF(PARTITION_VERIFY_NOT_AT_END());
        return &_node->_entry;
    }

    /** Returns a pointer to the Group associated with this iterator.
     **/
    template<typename M, typename V>
    inline const Group<M,V>* Group<M,V>::ConstIterator::getContainer() const
    {
        return _group;
    }

    /** Returns `true` if this Iterator and `i` point to the same Entry
     ** in the same Group list, and `false` otherwise.
     **
     ** \throw PartitionException if this Iterator and Iterator i are not
     ** associated with the same Group.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_ITERATOR_COMPARE
     ** \endcode
     **/
    template<typename M, typename V>
    inline bool Group<M,V>::ConstIterator::operator==(const Iterator& i) const
    {
        PARTITION_SAFETY_ITERATOR_COMPARE(PARTITION_VERIFY_COMPARE(i));
        return i._node == _node;
    }

    /** Returns `true` if this Iterator and `i` point to the same Entry
     ** in the same Group list, and `false` otherwise.
     **
     ** \throw PartitionException if this Iterator and Iterator i are not
     ** associated with the same Group.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_ITERATOR_COMPARE
     ** \endcode
     **/
    template<typename M, typename V>
    inline bool Group<M,V>::ConstIterator::operator==(const ConstIterator& i) const
    {
        PARTITION_SAFETY_ITERATOR_COMPARE(PARTITION_VERIFY_COMPARE(i));
        return i._node == _node;
    }

    /** Returns `false` if this Iterator and `i` point to the same Entry
     ** in the same Group list and `true` otherwise.
     **
     ** \throw PartitionException if this Iterator and Iterator i are not
     ** associated with the same Group.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_ITERATOR_COMPARE
     ** \endcode
     **/
    template<typename M, typename V>
    inline bool Group<M,V>::ConstIterator::operator!=(const Iterator& i) const
    {
        PARTITION_SAFETY_ITERATOR_COMPARE(PARTITION_VERIFY_COMPARE(i));
        return i._node != _node;
    }

    /** Returns `false` if this Iterator and `i` point to the same Entry
     ** in the same Group list and `true` otherwise.
     **
     ** \throw PartitionException if this Iterator and Iterator i are not
     ** associated with the same Group.\n\n
     ** This validation is only included if the following preprocessor
     ** directive evaluates to true during compilation:
     ** \code
     **     #if PARTITION_SL >= PARTITION_ST_ITERATOR_COMPARE
     ** \endcode
     **/
    template<typename M, typename V>
    inline bool Group<M,V>::ConstIterator::operator!=(const ConstIterator& i) const
    {
        PARTITION_SAFETY_ITERATOR_COMPARE(PARTITION_VERIFY_COMPARE(i));
        return i._node != _node;
    }

    template<typename M, typename V>
    inline Group<M,V>::ConstIterator::ConstIterator(const Group* group, const Node* node)
        :   _group(group),
            _node(node)
    {
    }

    template<typename M, typename V>
    inline void Group<M,V>::ConstIterator::_verifyGroup(const char* func, const char* file, int line) const
    {
        if(!_group)
            partitionThrow(func, file, line, "Iterator is not associated with a Group.");
    }

    template<typename M, typename V>
    inline void Group<M,V>::ConstIterator::_verifyNotAtEnd(const char* func, const char* file, int line) const
    {
        if(_node == &_group->_end)
            partitionThrow(func, file, line, "Iterator is before front or after back.");
    }

    template<typename M, typename V>
    inline void Group<M,V>::ConstIterator::_verifyCompare(const char* func, const char* file, int line, const Iterator& i) const
    {
        if(this->_group != i._group)
            partitionThrow(func, file, line, "Attempt to compare iterators from different groups.");
    }

    template<typename M, typename V>
    inline void Group<M,V>::ConstIterator::_verifyCompare(const char* func, const char* file, int line, const ConstIterator& i) const
    {
        if(this->_group != i._group)
            partitionThrow(func, file, line, "Attempt to compare iterators from different groups.");
    }
}

#endif
